@@ -35,20 +35,20 @@
 #   NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 #   MODIFICATIONS.
 #
-#   $Id: ldap.tcl,v 1.26 2008/11/22 12:25:27 mic42 Exp $
-#
 #   written by Jochen Loewer
 #   3 June, 1999
 #
 #-----------------------------------------------------------------------------
 
-package require Tcl 8.4
+package require Tcl 8.5
 package require asn 0.7
-package provide ldap 1.8
+package provide ldap 1.10.1
 
 namespace eval ldap {
 
     namespace export    connect secure_connect  \
+			starttls                \
+			tlsoptions              \
                         disconnect              \
                         bind unbind             \
                         bindSASL                \
@@ -66,9 +66,11 @@ namespace eval ldap {
 
     namespace import ::asn::*
 
-    variable SSLCertifiedAuthoritiesFile
     variable doDebug
 
+    # Valid TLS procotol versions
+    variable tlsProtocols [list -tls1 yes -tls1.1 yes -tls1.2 yes]
+
     set doDebug 0
 
     # LDAP result codes from the RFC
@@ -115,6 +117,49 @@ namespace eval ldap {
         80  other
     }
 
+    # TLS options for secure_connect and starttls
+    # (see tcltls documentation, function tls::import)
+    variable validTLSOptions
+    set validTLSOptions {
+	-cadir
+	-cafile
+	-certfile
+	-cipher
+	-command
+	-dhparams
+	-keyfile
+	-model
+	-password
+	-request
+	-require
+	-server
+	-servername
+	-ssl2
+	-ssl3
+	-tls1
+	-tls1.1
+	-tls1.2
+    }
+
+    # Default TLS options for secure_connect and starttls
+    variable defaultTLSOptions
+    array set defaultTLSOptions {
+	-request 1
+	-require 1
+	-ssl2    no
+	-ssl3    no
+	-tls1	 yes
+	-tls1.1	 yes
+	-tls1.2	 yes
+    }
+
+    variable curTLSOptions
+    array set curTLSOptions [array get defaultTLSOptions]
+
+    # are we using the old interface (TLSMode = "compatible") or the
+    # new one (TLSMode = "integrated")
+    variable TLSMode
+    set TLSMode "compatible"
 }
 
 
@@ -249,6 +294,30 @@ proc ldap::info_tls {args} {
    return $conn(tls)
 }
 
+#-----------------------------------------------------------------------------
+#   return the TLS connection status
+#
+#-----------------------------------------------------------------------------
+
+proc ldap::info_tlsstatus {args} {
+   if {[llength $args] != 1} {
+   	return -code error \
+	       "Wrong # of arguments. Usage: ldap::info tlsstatus handle"
+   }
+   CheckHandle [lindex $args 0]
+   upvar #0 [lindex $args 0] conn
+   if {![::info exists conn(tls)]} {
+   	return -code error \
+		"\"[lindex $args 0]\" is not a ldap connection handle"
+   }
+   if {$conn(tls)} then {
+       set r [::tls::status $conn(sock)]
+   } else {
+       set r {}
+   }
+   return $r
+}
+
 proc ldap::info_saslmechanisms {args} {
    if {[llength $args] != 1} {
    	return -code error \
@@ -329,7 +398,7 @@ proc ldap::Whoami {handle} {
             "Server does not support the \"Who am I?\" extension"
     }
 
-    set request [asnApplicationConstr 23 [asnOctetString 1.3.6.1.4.1.4203.1.11.3]]
+    set request [asnApplicationConstr 23 [asnEncodeString 80 1.3.6.1.4.1.4203.1.11.3]]
     set mid [SendMessage $handle $request]
     set response [WaitForResponse $handle $mid]
 
@@ -389,48 +458,106 @@ proc ldap::connect { host {port 389} } {
     return ::ldap::ldap$sock
 }
 
+#-----------------------------------------------------------------------------
+#    tlsoptions
+#
+#-----------------------------------------------------------------------------
+proc ldap::tlsoptions {args} {
+    variable curTLSOptions
+    variable validTLSOptions
+    variable defaultTLSOptions
+    variable TLSMode
+
+    if {$args eq "reset"} then {
+	array set curTLSOptions [array get defaultTLSOptions]
+    } else {
+	foreach {opt val} $args {
+	    if {$opt in $validTLSOptions} then {
+		set curTLSOptions($opt) $val
+	    } else {
+		return -code error "invalid TLS option '$opt'"
+	    }
+	}
+    }
+    set TLSMode "integrated"
+    return [array get curTLSOptions]
+}
+
 #-----------------------------------------------------------------------------
 #    secure_connect
 #
 #-----------------------------------------------------------------------------
-proc ldap::secure_connect { host {port 636} } {
+proc ldap::secure_connect { host {port 636} {verify_cert ""} {sni_servername ""}} {
 
-    variable SSLCertifiedAuthoritiesFile
+    variable tlsProtocols
+    variable curTLSOptions
+    variable TLSMode
 
     package require tls
 
     #------------------------------------------------------------------
-    #   connect via TCP/IP
+    #   set options
     #------------------------------------------------------------------
-    set sock [socket $host $port]
-    fconfigure $sock -blocking no -translation binary -buffering full
+
+    if {$TLSMode eq "compatible"} then {
+	#
+	# Compatible with old mode. Build a TLS socket with appropriate
+	# parameters, without changing any other parameter which may
+	# have been set by a previous call to tls::init (as specified
+	# in the ldap.tcl manpage).
+	#
+	if {$verify_cert eq ""} then {
+	    set verify_cert 1
+	}
+	set cmd [list tls::socket -request 1 -require $verify_cert \
+				  -ssl2 no -ssl3 no]
+	if {$sni_servername ne ""} {
+	    lappend cmd -servername $sni_servername
+	}
+
+	# The valid ones depend on the server and openssl version,
+	# tls::ciphers all tells it in the error message, but offers no
+	# nice introspection.
+	foreach {proto active} $tlsProtocols {
+	    lappend cmd $proto $active
+	}
+
+	lappend cmd $host $port
+    } else {
+	#
+	# New, integrated mode. Use only parameters set with
+	# ldap::tlsoptions to build the socket.
+	#
+
+	if {$verify_cert ne "" || $sni_servername ne ""} then {
+	    return -code error "verify_cert/sni_servername: incompatible with the use of tlsoptions"
+	}
+
+	set cmd [list tls::socket {*}[array get curTLSOptions] $host $port]
+    }
 
     #------------------------------------------------------------------
-    #   make it a SSL connection
+    #   connect via TCP/IP
+    #------------------------------------------------------------------
+
+    set sock [eval $cmd]
+
+    #------------------------------------------------------------------
+    #   Run the TLS handshake
     #
     #------------------------------------------------------------------
-    #tls::import $sock -cafile $SSLCertifiedAuthoritiesFile -ssl2 no -ssl3 yes -tls1 yes
-    tls::import $sock -cafile "" -certfile "" -keyfile "" \
-                      -request 1 -server 0 -require 0 -ssl2 no -ssl3 yes -tls1 yes
-    set retry 0
-    while {1} {
-        if {$retry > 20} {
-            close $sock
-            return -code error "too long retry to setup SSL connection"
-        }
-        if {[catch { tls::handshake $sock } err]} {
-            if {[string match "*resource temporarily unavailable*" $err]} {
-                after 50
-                incr retry
-            } else {
-                close $sock
-                return -code error $err
-            }
-        } else {
-            break
-        }
+    
+    # run the handshake in synchronous I/O mode
+    fconfigure $sock -blocking yes -translation binary -buffering full
+
+    if {[catch { tls::handshake $sock } err]} {
+	close $sock
+	return -code error $err
     }
 
+    # from now on, run in asynchronous I/O mode
+    fconfigure $sock -blocking no -translation binary -buffering full
+
     #--------------------------------------
     #   initialize connection array
     #--------------------------------------
@@ -458,11 +585,62 @@ proc ldap::secure_connect { host {port 636} } {
 #    starttls -  negotiate tls on an open ldap connection
 #
 #------------------------------------------------------------------------------
-proc ldap::starttls {handle {cafile ""} {certfile ""} {keyfile ""}} {
+proc ldap::starttls {handle {cafile ""} {certfile ""} {keyfile ""} \
+                     {verify_cert ""} {sni_servername ""}} {
+    variable tlsProtocols
+    variable curTLSOptions
+    variable TLSMode
+
     CheckHandle $handle
 
     upvar #0 $handle conn
 
+    #------------------------------------------------------------------
+    #   set options
+    #------------------------------------------------------------------
+
+    if {$TLSMode eq "compatible"} then {
+	#
+	# Compatible with old mode. Build a TLS socket with appropriate
+	# parameters, without changing any other parameter which may
+	# have been set by a previous call to tls::init (as specified
+	# in the ldap.tcl manpage).
+	#
+	if {$verify_cert eq ""} then {
+	    set verify_cert 1
+	}
+	set cmd [list tls::import $conn(sock) \
+		     -cafile $cafile -certfile $certfile -keyfile $keyfile \
+		     -request 1 -server 0 -require $verify_cert \
+		     -ssl2 no -ssl3 no ]
+	if {$sni_servername ne ""} {
+	    lappend cmd -servername $sni_servername
+	}
+
+	# The valid ones depend on the server and openssl version,
+	# tls::ciphers all tells it in the error message, but offers no
+	# nice introspection.
+	foreach {proto active} $tlsProtocols {
+	    lappend cmd $proto $active
+	}
+    } else {
+	#
+	# New, integrated mode. Use only parameters set with
+	# ldap::tlsoptions to build the socket.
+	#
+
+	if {$cafile ne "" || $certfile ne "" || $keyfile ne "" ||
+		$verify_cert ne "" || $sni_servername ne ""} then {
+	    return -code error "cafile/certfile/keyfile/verify_cert/sni_servername: incompatible with the use of tlsoptions"
+	}
+
+	set cmd [list tls::import $conn(sock) {*}[array get curTLSOptions]]
+    }
+
+    #------------------------------------------------------------------
+    #   check handle
+    #------------------------------------------------------------------
+
     if {$conn(tls)} {
         return -code error \
             "Cannot StartTLS on connection, TLS already running"
@@ -485,7 +663,7 @@ proc ldap::starttls {handle {cafile ""} {certfile ""} {keyfile ""}} {
     package require tls
 
 
-    set request [asnApplicationConstr 23 [asnOctetString 1.3.6.1.4.1.1466.20037]]
+    set request [asnApplicationConstr 23 [asnEncodeString 80 1.3.6.1.4.1.1466.20037]]
     set mid [SendMessage $handle $request]
     set conn(tlsHandshakeInProgress) 1
     set response [WaitForResponse $handle $mid]
@@ -517,8 +695,10 @@ proc ldap::starttls {handle {cafile ""} {certfile ""} {keyfile ""}} {
             "Unexpected LDAP response"
     }
 
-    tls::import $conn(sock) -cafile $cafile -certfile $certfile -keyfile $keyfile \
-                      -request 1 -server 0 -require 0 -ssl2 no -ssl3 yes -tls1 yes
+    # Initiate the TLS socket setup
+
+    eval $cmd
+
     set retry 0
     while {1} {
         if {$retry > 20} {
@@ -738,8 +918,12 @@ proc ldap::MessageReceiver {handle} {
                     set conn(pdu,partial) 1
                     append conn(pdu,received) $type
                 }
-                }
-            eof {
+	    }
+	    partial {
+		# See ticket https://core.tcl.tk/tcllib/tktview/c247ed5db42e373470bf8a6302717e76eb3c6106
+		return
+	    }
+	    eof {
                 CleanupWaitingMessages $handle
                 set conn(lastError) "Server closed connection"
                 catch {close $conn(sock)}
@@ -750,13 +934,12 @@ proc ldap::MessageReceiver {handle} {
                 set bytes $type[read $conn(sock)]
                 binary scan $bytes h* values
                 set conn(lastError) [format \
-                    "Error reading SEQUENCE response for handle %s : %s : %s" $handle $code $values]
+					 "Error reading SEQUENCE response for handle %s : %s : %s" $handle $code $values]
                 return
-                }
+	    }
         }
     }
 
-
     # fetch the length
     if {[::info exists conn(pdu,length)] && $conn(pdu,length) >= 0} {
         # we already have a decoded length
