@@ -12,7 +12,7 @@
 
 package require Tcl 8.5;# lrepeat
 package require math::interpolate
-package provide math::calculus 0.8.2
+package provide math::calculus 1.0
 
 # math::calculus --
 #    Namespace for the commands
@@ -37,7 +37,7 @@ namespace eval ::math::calculus {
     namespace export romberg_powerLawLower romberg_powerLawUpper
     namespace export romberg_expLower romberg_expUpper
 
-    namespace export regula_falsi
+    namespace export regula_falsi root_bisection root_secant root_brent root_chandrupatla
 
     variable nr_maxiter    20
     variable nr_tolerance   0.001
@@ -708,7 +708,7 @@ proc ::math::calculus::midpoint { f a b { n 0 } { s 0. } } {
 #----------------------------------------------------------------------
 #
 # romberg --
-#	
+#
 #	Compute the integral of a function over an interval using
 #	Romberg's method.
 #
@@ -1139,7 +1139,7 @@ proc ::math::calculus::romberg_powerLawLower { gamma f a b args } {
 	       [expr { $gamma / $onemgamma }] \
 	       [expr { 1 / $onemgamma }] \
 	       $a]
-	
+
     set limit [expr { pow( $b - $a, $onemgamma ) }]
     set result {}
     foreach v [eval [linsert $args 0 romberg $f 0 $limit]] {
@@ -1258,7 +1258,7 @@ proc ::math::calculus::romberg_powerLawUpper { gamma f a b args } {
 	       [expr { $gamma / $onemgamma }] \
 	       [expr { 1. / $onemgamma }] \
 	       $b]
-	
+
     set limit [expr { pow( $b - $a, $onemgamma ) }]
     set result {}
     foreach v [eval [linsert $args 0 romberg $f 0 $limit]] {
@@ -1643,3 +1643,8 @@ proc ::math::calculus::qk15_detailed {xstart xend func {n 1}} {
 
     return [list $result $abserr $resabs $resasc]
 }
+
+#
+# Add the root finding procedures based on bracketing a root
+#
+source [file join [file dirname [info script]] rootfind.tcl]
