@@ -198,100 +198,100 @@ if {![catch {package require Trf 2.0}] && ![catch {::md5 -- test}]} {
 	    # Let [abcd k s i] denote the operation
 	    #      a = b + ((a + F(b,c,d) + X[k] + T[i]) <<< s).
 	    # [ABCD  0  7  1]  [DABC  1 12  2]  [CDAB  2 17  3]  [BCDA  3 22  4]
-	    set A [expr {$B + [<<< [expr {$A + [F $B $C $D] + $X0  + $T01}]  7]}]
-	    set D [expr {$A + [<<< [expr {$D + [F $A $B $C] + $X1  + $T02}] 12]}]
-	    set C [expr {$D + [<<< [expr {$C + [F $D $A $B] + $X2  + $T03}] 17]}]
-	    set B [expr {$C + [<<< [expr {$B + [F $C $D $A] + $X3  + $T04}] 22]}]
+	    set A [expr {($B + [<<< [expr {$A + [F $B $C $D] + $X0  + $T01}]  7]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [F $A $B $C] + $X1  + $T02}] 12]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [F $D $A $B] + $X2  + $T03}] 17]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [F $C $D $A] + $X3  + $T04}] 22]) & 0xffffffff}]
 	    # [ABCD  4  7  5]  [DABC  5 12  6]  [CDAB  6 17  7]  [BCDA  7 22  8]
-	    set A [expr {$B + [<<< [expr {$A + [F $B $C $D] + $X4  + $T05}]  7]}]
-	    set D [expr {$A + [<<< [expr {$D + [F $A $B $C] + $X5  + $T06}] 12]}]
-	    set C [expr {$D + [<<< [expr {$C + [F $D $A $B] + $X6  + $T07}] 17]}]
-	    set B [expr {$C + [<<< [expr {$B + [F $C $D $A] + $X7  + $T08}] 22]}]
+	    set A [expr {($B + [<<< [expr {$A + [F $B $C $D] + $X4  + $T05}]  7]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [F $A $B $C] + $X5  + $T06}] 12]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [F $D $A $B] + $X6  + $T07}] 17]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [F $C $D $A] + $X7  + $T08}] 22]) & 0xffffffff}]
 	    # [ABCD  8  7  9]  [DABC  9 12 10]  [CDAB 10 17 11]  [BCDA 11 22 12]
-	    set A [expr {$B + [<<< [expr {$A + [F $B $C $D] + $X8  + $T09}]  7]}]
-	    set D [expr {$A + [<<< [expr {$D + [F $A $B $C] + $X9  + $T10}] 12]}]
-	    set C [expr {$D + [<<< [expr {$C + [F $D $A $B] + $X10 + $T11}] 17]}]
-	    set B [expr {$C + [<<< [expr {$B + [F $C $D $A] + $X11 + $T12}] 22]}]
+	    set A [expr {($B + [<<< [expr {$A + [F $B $C $D] + $X8  + $T09}]  7]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [F $A $B $C] + $X9  + $T10}] 12]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [F $D $A $B] + $X10 + $T11}] 17]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [F $C $D $A] + $X11 + $T12}] 22]) & 0xffffffff}]
 	    # [ABCD 12  7 13]  [DABC 13 12 14]  [CDAB 14 17 15]  [BCDA 15 22 16]
-	    set A [expr {$B + [<<< [expr {$A + [F $B $C $D] + $X12 + $T13}]  7]}]
-	    set D [expr {$A + [<<< [expr {$D + [F $A $B $C] + $X13 + $T14}] 12]}]
-	    set C [expr {$D + [<<< [expr {$C + [F $D $A $B] + $X14 + $T15}] 17]}]
-	    set B [expr {$C + [<<< [expr {$B + [F $C $D $A] + $X15 + $T16}] 22]}]
+	    set A [expr {($B + [<<< [expr {$A + [F $B $C $D] + $X12 + $T13}]  7]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [F $A $B $C] + $X13 + $T14}] 12]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [F $D $A $B] + $X14 + $T15}] 17]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [F $C $D $A] + $X15 + $T16}] 22]) & 0xffffffff}]
 
 	    # Round 2.
 	    # Let [abcd k s i] denote the operation
 	    #      a = b + ((a + G(b,c,d) + X[k] + T[i]) <<< s).
 	    # Do the following 16 operations.
 	    # [ABCD  1  5 17]  [DABC  6  9 18]  [CDAB 11 14 19]  [BCDA  0 20 20]
-	    set A [expr {$B + [<<< [expr {$A + [G $B $C $D] + $X1  + $T17}]  5]}]
-	    set D [expr {$A + [<<< [expr {$D + [G $A $B $C] + $X6  + $T18}]  9]}]
-	    set C [expr {$D + [<<< [expr {$C + [G $D $A $B] + $X11 + $T19}] 14]}]
-	    set B [expr {$C + [<<< [expr {$B + [G $C $D $A] + $X0  + $T20}] 20]}]
+	    set A [expr {($B + [<<< [expr {$A + [G $B $C $D] + $X1  + $T17}]  5]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [G $A $B $C] + $X6  + $T18}]  9]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [G $D $A $B] + $X11 + $T19}] 14]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [G $C $D $A] + $X0  + $T20}] 20]) & 0xffffffff}]
 	    # [ABCD  5  5 21]  [DABC 10  9 22]  [CDAB 15 14 23]  [BCDA  4 20 24]
-	    set A [expr {$B + [<<< [expr {$A + [G $B $C $D] + $X5  + $T21}]  5]}]
-	    set D [expr {$A + [<<< [expr {$D + [G $A $B $C] + $X10 + $T22}]  9]}]
-	    set C [expr {$D + [<<< [expr {$C + [G $D $A $B] + $X15 + $T23}] 14]}]
-	    set B [expr {$C + [<<< [expr {$B + [G $C $D $A] + $X4  + $T24}] 20]}]
+	    set A [expr {($B + [<<< [expr {$A + [G $B $C $D] + $X5  + $T21}]  5]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [G $A $B $C] + $X10 + $T22}]  9]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [G $D $A $B] + $X15 + $T23}] 14]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [G $C $D $A] + $X4  + $T24}] 20]) & 0xffffffff}]
 	    # [ABCD  9  5 25]  [DABC 14  9 26]  [CDAB  3 14 27]  [BCDA  8 20 28]
-	    set A [expr {$B + [<<< [expr {$A + [G $B $C $D] + $X9  + $T25}]  5]}]
-	    set D [expr {$A + [<<< [expr {$D + [G $A $B $C] + $X14 + $T26}]  9]}]
-	    set C [expr {$D + [<<< [expr {$C + [G $D $A $B] + $X3  + $T27}] 14]}]
-	    set B [expr {$C + [<<< [expr {$B + [G $C $D $A] + $X8  + $T28}] 20]}]
+	    set A [expr {($B + [<<< [expr {$A + [G $B $C $D] + $X9  + $T25}]  5]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [G $A $B $C] + $X14 + $T26}]  9]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [G $D $A $B] + $X3  + $T27}] 14]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [G $C $D $A] + $X8  + $T28}] 20]) & 0xffffffff}]
 	    # [ABCD 13  5 29]  [DABC  2  9 30]  [CDAB  7 14 31]  [BCDA 12 20 32]
-	    set A [expr {$B + [<<< [expr {$A + [G $B $C $D] + $X13 + $T29}]  5]}]
-	    set D [expr {$A + [<<< [expr {$D + [G $A $B $C] + $X2  + $T30}]  9]}]
-	    set C [expr {$D + [<<< [expr {$C + [G $D $A $B] + $X7  + $T31}] 14]}]
-	    set B [expr {$C + [<<< [expr {$B + [G $C $D $A] + $X12 + $T32}] 20]}]
+	    set A [expr {($B + [<<< [expr {$A + [G $B $C $D] + $X13 + $T29}]  5]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [G $A $B $C] + $X2  + $T30}]  9]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [G $D $A $B] + $X7  + $T31}] 14]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [G $C $D $A] + $X12 + $T32}] 20]) & 0xffffffff}]
 
 	    # Round 3.
 	    # Let [abcd k s t] [sic] denote the operation
 	    #     a = b + ((a + H(b,c,d) + X[k] + T[i]) <<< s).
 	    # Do the following 16 operations.
 	    # [ABCD  5  4 33]  [DABC  8 11 34]  [CDAB 11 16 35]  [BCDA 14 23 36]
-	    set A [expr {$B + [<<< [expr {$A + [H $B $C $D] + $X5  + $T33}]  4]}]
-	    set D [expr {$A + [<<< [expr {$D + [H $A $B $C] + $X8  + $T34}] 11]}]
-	    set C [expr {$D + [<<< [expr {$C + [H $D $A $B] + $X11 + $T35}] 16]}]
-	    set B [expr {$C + [<<< [expr {$B + [H $C $D $A] + $X14 + $T36}] 23]}]
+	    set A [expr {($B + [<<< [expr {$A + [H $B $C $D] + $X5  + $T33}]  4]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [H $A $B $C] + $X8  + $T34}] 11]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [H $D $A $B] + $X11 + $T35}] 16]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [H $C $D $A] + $X14 + $T36}] 23]) & 0xffffffff}]
 	    # [ABCD  1  4 37]  [DABC  4 11 38]  [CDAB  7 16 39]  [BCDA 10 23 40]
-	    set A [expr {$B + [<<< [expr {$A + [H $B $C $D] + $X1  + $T37}]  4]}]
-	    set D [expr {$A + [<<< [expr {$D + [H $A $B $C] + $X4  + $T38}] 11]}]
-	    set C [expr {$D + [<<< [expr {$C + [H $D $A $B] + $X7  + $T39}] 16]}]
-	    set B [expr {$C + [<<< [expr {$B + [H $C $D $A] + $X10 + $T40}] 23]}]
+	    set A [expr {($B + [<<< [expr {$A + [H $B $C $D] + $X1  + $T37}]  4]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [H $A $B $C] + $X4  + $T38}] 11]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [H $D $A $B] + $X7  + $T39}] 16]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [H $C $D $A] + $X10 + $T40}] 23]) & 0xffffffff}]
 	    # [ABCD 13  4 41]  [DABC  0 11 42]  [CDAB  3 16 43]  [BCDA  6 23 44]
-	    set A [expr {$B + [<<< [expr {$A + [H $B $C $D] + $X13 + $T41}]  4]}]
-	    set D [expr {$A + [<<< [expr {$D + [H $A $B $C] + $X0  + $T42}] 11]}]
-	    set C [expr {$D + [<<< [expr {$C + [H $D $A $B] + $X3  + $T43}] 16]}]
-	    set B [expr {$C + [<<< [expr {$B + [H $C $D $A] + $X6  + $T44}] 23]}]
+	    set A [expr {($B + [<<< [expr {$A + [H $B $C $D] + $X13 + $T41}]  4]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [H $A $B $C] + $X0  + $T42}] 11]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [H $D $A $B] + $X3  + $T43}] 16]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [H $C $D $A] + $X6  + $T44}] 23]) & 0xffffffff}]
 	    # [ABCD  9  4 45]  [DABC 12 11 46]  [CDAB 15 16 47]  [BCDA  2 23 48]
-	    set A [expr {$B + [<<< [expr {$A + [H $B $C $D] + $X9  + $T45}]  4]}]
-	    set D [expr {$A + [<<< [expr {$D + [H $A $B $C] + $X12 + $T46}] 11]}]
-	    set C [expr {$D + [<<< [expr {$C + [H $D $A $B] + $X15 + $T47}] 16]}]
-	    set B [expr {$C + [<<< [expr {$B + [H $C $D $A] + $X2  + $T48}] 23]}]
+	    set A [expr {($B + [<<< [expr {$A + [H $B $C $D] + $X9  + $T45}]  4]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [H $A $B $C] + $X12 + $T46}] 11]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [H $D $A $B] + $X15 + $T47}] 16]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [H $C $D $A] + $X2  + $T48}] 23]) & 0xffffffff}]
 
 	    # Round 4.
 	    # Let [abcd k s t] [sic] denote the operation
 	    #     a = b + ((a + I(b,c,d) + X[k] + T[i]) <<< s).
 	    # Do the following 16 operations.
 	    # [ABCD  0  6 49]  [DABC  7 10 50]  [CDAB 14 15 51]  [BCDA  5 21 52]
-	    set A [expr {$B + [<<< [expr {$A + [I $B $C $D] + $X0  + $T49}]  6]}]
-	    set D [expr {$A + [<<< [expr {$D + [I $A $B $C] + $X7  + $T50}] 10]}]
-	    set C [expr {$D + [<<< [expr {$C + [I $D $A $B] + $X14 + $T51}] 15]}]
-	    set B [expr {$C + [<<< [expr {$B + [I $C $D $A] + $X5  + $T52}] 21]}]
+	    set A [expr {($B + [<<< [expr {$A + [I $B $C $D] + $X0  + $T49}]  6]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [I $A $B $C] + $X7  + $T50}] 10]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [I $D $A $B] + $X14 + $T51}] 15]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [I $C $D $A] + $X5  + $T52}] 21]) & 0xffffffff}]
 	    # [ABCD 12  6 53]  [DABC  3 10 54]  [CDAB 10 15 55]  [BCDA  1 21 56]
-	    set A [expr {$B + [<<< [expr {$A + [I $B $C $D] + $X12 + $T53}]  6]}]
-	    set D [expr {$A + [<<< [expr {$D + [I $A $B $C] + $X3  + $T54}] 10]}]
-	    set C [expr {$D + [<<< [expr {$C + [I $D $A $B] + $X10 + $T55}] 15]}]
-	    set B [expr {$C + [<<< [expr {$B + [I $C $D $A] + $X1  + $T56}] 21]}]
+	    set A [expr {($B + [<<< [expr {$A + [I $B $C $D] + $X12 + $T53}]  6]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [I $A $B $C] + $X3  + $T54}] 10]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [I $D $A $B] + $X10 + $T55}] 15]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [I $C $D $A] + $X1  + $T56}] 21]) & 0xffffffff}]
 	    # [ABCD  8  6 57]  [DABC 15 10 58]  [CDAB  6 15 59]  [BCDA 13 21 60]
-	    set A [expr {$B + [<<< [expr {$A + [I $B $C $D] + $X8  + $T57}]  6]}]
-	    set D [expr {$A + [<<< [expr {$D + [I $A $B $C] + $X15 + $T58}] 10]}]
-	    set C [expr {$D + [<<< [expr {$C + [I $D $A $B] + $X6  + $T59}] 15]}]
-	    set B [expr {$C + [<<< [expr {$B + [I $C $D $A] + $X13 + $T60}] 21]}]
+	    set A [expr {($B + [<<< [expr {$A + [I $B $C $D] + $X8  + $T57}]  6]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [I $A $B $C] + $X15 + $T58}] 10]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [I $D $A $B] + $X6  + $T59}] 15]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [I $C $D $A] + $X13 + $T60}] 21]) & 0xffffffff}]
 	    # [ABCD  4  6 61]  [DABC 11 10 62]  [CDAB  2 15 63]  [BCDA  9 21 64]
-	    set A [expr {$B + [<<< [expr {$A + [I $B $C $D] + $X4  + $T61}]  6]}]
-	    set D [expr {$A + [<<< [expr {$D + [I $A $B $C] + $X11 + $T62}] 10]}]
-	    set C [expr {$D + [<<< [expr {$C + [I $D $A $B] + $X2  + $T63}] 15]}]
-	    set B [expr {$C + [<<< [expr {$B + [I $C $D $A] + $X9  + $T64}] 21]}]
+	    set A [expr {($B + [<<< [expr {$A + [I $B $C $D] + $X4  + $T61}]  6]) & 0xffffffff}]
+	    set D [expr {($A + [<<< [expr {$D + [I $A $B $C] + $X11 + $T62}] 10]) & 0xffffffff}]
+	    set C [expr {($D + [<<< [expr {$C + [I $D $A $B] + $X2  + $T63}] 15]) & 0xffffffff}]
+	    set B [expr {($C + [<<< [expr {$B + [I $C $D $A] + $X9  + $T64}] 21]) & 0xffffffff}]
 
 	    # Then perform the following additions. (That is increment each
 	    #   of the four registers by the value it had before this block
@@ -355,7 +355,8 @@ if {![catch {package require Trf 2.0}] && ![catch {::md5 -- test}]} {
 	regsub -all -- {\[ *<<< +\[ *expr +({[^\}]*})\] +([0-9]+) *\]} $md5body {(([set x [expr \1]] << \2) |  (($x >> R\2) \& S\2))} md5body
 
 	# now replace the R and S
-	set map {}
+	variable map {}
+	variable i
 	foreach i { 
 	    7 12 17 22
 	    5  9 14 20
@@ -366,6 +367,8 @@ if {![catch {package require Trf 2.0}] && ![catch {::md5 -- test}]} {
 	}
 	
 	# inline the values of T
+	variable tVal
+	variable tName
 	foreach \
 		tName {
 	    T01 T02 T03 T04 T05 T06 T07 T08 T09 T10 
@@ -405,7 +408,7 @@ if {![catch {package require Trf 2.0}] && ![catch {::md5 -- test}]} {
 	proc md5 {msg} $md5body
 
 	# unset auxiliary variables
-	unset md5body tName tVal map
+	unset md5body tName tVal map i
     }
 
     proc ::md5::byte0 {i} {expr {0xff & $i}}
@@ -451,4 +454,4 @@ if {![catch {package require Trf 2.0}] && ![catch {::md5 -- test}]} {
     }
 }
 
-package provide md5 1.4.4
+package provide md5 1.4.5
